"""
    aiotinydb - asyncio compatibility shim for tinydb

    Copyright 2017 Pavel Pletenev <cpp.create@gmail.com>
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""


class AIOTinyDBError(Exception):
    'Base class for AIOTinyDB exceptions'
    pass


class DatabaseNotReady(AIOTinyDBError):
    'Indicates wrong usage of AIOTinyDB'
    pass


class ReadonlyStorageError(AIOTinyDBError):
    'Raised when a readonly storage is attempted to be written to'
    pass


class NotOverridableError(AIOTinyDBError):
    'Indicates a non-overridable method'
    pass
